/*
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

#pragma once

#include "physical_operator.h"
#include "types.h"


namespace ppds {
    class ProjectionOperatorState : public ONCIteratorState {
    public:
        Record child_output;
    };

    class PhysicalProjection : public PhysicalOperator {
    public:
        PhysicalProjection(std::shared_ptr<PhysicalOperator> child, std::vector<uint16_t> column_ids_p) :
                column_ids(std::move(column_ids_p)) {
            children.push_back(std::move(child));
        }

        void Open() override {
            auto proj_state = std::make_shared<ProjectionOperatorState>();
            state = proj_state;
            children[0]->Open();
        };

        void Next(Record& result) override {
            auto& proj_state = (ProjectionOperatorState&) *state;
            children[0]->Next(proj_state.child_output);
            if (!proj_state.child_output.IsValid()) {
                result.Clear();
                return;
            }

            if (!result.IsValid()) {
                auto types = std::make_shared<std::vector<DataType>>();
                for (auto column_id: column_ids) {
                    types->push_back(proj_state.child_output.GetColumnType(column_id));
                }
                result.Initialize(types);
            }

            for (uint16_t i = 0; i < column_ids.size(); i++) {
                result.SetValue(i, proj_state.child_output.GetValue(column_ids[i]));
            }
        }

        void Close() override {
            state = nullptr;
            children[0]->Close();
        }

    private:
        const std::vector<uint16_t> column_ids;
    };
}
